#!/usr/bin/perl

# Read a text file and make it a frameset HTML document set where
# every word is linked to its gloss in the gzb lexicon (shown in the
# lower frame).

# (c) 2007-2012 Jim Henry.  Creative Commons licensing (not that it will
# be useful to anybody else as-is, but feel free to copy and modify)

###TODO: if the next word in the paragraph is {pxej}, then don't report
# 	this word or any morpheme within it as a misspelling/unknown word.

###TODO: if we can find a match by stripping out any incorporated 
#  pronouns, then don't report an unknown word.

use strict;
use gzb_ascii_unicode;

sub usage;
sub write_frameset( $$ );
sub make_hyperlink( $ );

my %unknown_roots;
my %unknown_compounds;
my $path = "";
my $infile;
my $outfile_glosslinks;
my $outfile_frameset;

# default arg value unless overridden with -l option
my $lexicon = "nxcgtx.rdb";
my $bottomframefile = "nxcgtx.htm";

# default document title unless overriden by -t option
my $title = "gloss-links file generated by gen-glossed-doc-frameset.pl";
my $arg;

# read command line args
###TODO rewrite to use GetOpt::Long
while ( $arg = shift ) {
    if ( $arg =~ m/^-[?h]/ )  { 
	&usage;
	exit;
    } elsif ( $arg eq "-o" ) {  ## output filename
	my $outbase = shift;
	$outfile_frameset = $outbase . ".html";
	$outfile_glosslinks = $outbase . "_f.html";
    } elsif ( $arg eq "-l" ) { ## lexicon
	$lexicon = shift;
    } elsif ( $arg eq "-t" ) { ## document title - all of command line from here on becomes part of title
	$title = gzb_ascii_unicode_convert( join( " ", @ARGV ) );
	last;
    } else {
	if ( not defined $infile ) {
	    $infile = $arg;
	} else {
	    &usage;
	    exit;
	}
    }
}

if ( not defined $infile ) {
    &usage;
    exit;
}


# set output file names based on input file name or other args overriding it

if ( not defined $outfile_frameset ) {
    my $infile_base = $infile;
    $infile_base =~ s/\.txt$//;
    if ( $infile_base =~ m(^(.*)/(.*)$) ) {
	$path = $1 . "/";
	$infile_base = $2;
	print STDERR "note: input file not in current dir, but writing output file to current dir anyway\n";

    }
    $outfile_frameset = $infile_base . ".html";
    $outfile_glosslinks = $infile_base . "_f.html";
}

# slurp in lexicon file, filtering out unwanted lines
open LEXICON, $lexicon  or die "can't open $lexicon for reading\n";
my @lexicon_words;
while ( <LEXICON> ) {
    my @f = split /\t/;
    if ( $f[1] =~ /[bdfkmnprs]/ ) {
	push @lexicon_words, $f[0];
    }
}
close LEXICON;

open INFILE, $infile 	or die "can't open $infile for reading\n"; 
open GLOSSFRAME, "> $outfile_glosslinks"  or die "can't open $outfile_glosslinks for writing\n";


&write_frameset( $outfile_frameset, $outfile_glosslinks );

# write HTML header to text frame
print GLOSSFRAME<<HEADER;
<HTML>
<HEAD>
    <meta name="generator" content="gen-glossed-doc-frameset.pl">
    <meta name="viewport" content="width=device-width, initial-scale=1.0, user-scalable=yes" />
<TITLE>
$title
</TITLE>
</HEAD>
</HTML>
<BODY>
<H1>
$title
</H1>

<p> 
Click the hyperlinked words and morphemes to get their gloss in
the lower frame.  (Links are not underlined here as underlining almost
every word on the page is cluttery and makes characters with
descenders hard to distinguish.)

HEADER

my $in_para = 0;
my $p = 0;
my $outline = "";
my $linkcount = 0;
my $word;
my $line = 0;

while ( <INFILE> ) {
    $line++;
    # blank line: end of paragraph
    if ( /^\s*$/ && $in_para ) {
	print GLOSSFRAME "</p>\n\n";
	$in_para = 0;
    }

    # if line is not blank
    if ( ! /^\s*$/ ) {

	if ( 0 == $in_para ) {
	    print  GLOSSFRAME  "<p id=\"p" . ++$p . "\">\n";
	    $in_para = 1;
	}
	
	my $origline = $_;  ##DEBUG
	$outline = "";

	###TODO put this and other gzb split regexes from various
	# scripts into a module with functions for splitting on word
	# vs. morpheme boundaries and saving or throwing away the
	# delimiters... want consistent behavior across all scripts 
	# and ability to fix bugs in one place and not have to search for
	# other regexes with same bug.

        # split line, saving delimiters and words in alternate result fields
	my @words = split(/([\s!?.{}\[\]\(\),\"\*;:~]+)/, $_ );

	# replace < > " & with HTML char entities -- where?

	for ( my $i = 0; $i < scalar(@words); $i++) {
	    ##print "line $line word $i: " . $words[$i] . "\n";   ##DEBUG

	    $word = $words[$i];
	    if ( grep { $word eq $_ } @lexicon_words ) {
                $word = &make_hyperlink( $word );
            } elsif ( $word =~ /-/ ) {
		if ( $word =~ /[a-z][a-z]/ ) {
		    $unknown_compounds{ $word } = 1;
		}
	        # if no match found and word contains '-'
		my @morphemes = split( /-/, $word );
		my $linkedword = "";
		my $j = 0;

		for ( $j = 0; $j < scalar(@morphemes); $j++ ) {
		    ##print "line $line word $i morpheme $j: " . $morphemes[$j] . "\n";   ##DEBUG
		    # check for match in nxcgtx array
		    if ( grep { $morphemes[$j] eq $_ } @lexicon_words ) {
			$linkedword .= &make_hyperlink( $morphemes[$j] );
		    } else {
			# if this one is unknown, and there isn't a proper name suffix elsewhere in the word
			if ( $morphemes[$j] =~ /^[a-z']+$/ 
			     && not grep { $_ =~ m/^(r|sq|w|l|g)am$/ } @morphemes )
			     #&& $morphemes[ $#morphemes ] !~ m/^(r|sq|w|l|g)am$/ 
			{
			    $unknown_roots{ $morphemes[$j] } = 1;
			}
			$linkedword .= $morphemes[$j];
		    }
		    if ( $j < (scalar(@morphemes) - 1) ) {
			$linkedword .= "-";
		    }
		} ## end for each morpheme in compound

=pod Unfinished experiment with building it up in reverse order...
		my $last_morpheme = "";
		for ( $j = $#morphemes; $j >= 0; $j-- ) {
		    # check for match in nxcgtx array
		    if ( grep { $morphemes[$j] eq $_ } @lexicon_words ) {
			$linkedword = &make_hyperlink( $morphemes[$j] ) . $linkedword;
		    } else {
			if ( $morphemes[$j] =~ /[a-z][a-z]/ ) {
			    $unknown_roots{ $morphemes[$j] } = 1;
			}
			$linkedword = $morphemes[$j]  . $linkedword;
		    }
		    if ( $j < (scalar(@morphemes) - 1) ) {
			$linkedword = "-" . $linkedword;
		    }
		} ## end for each morpheme in compound
=cut
		$word = $linkedword;
            } else {  # an unknown non-hyphenated word
		# if it's not numbers or all caps,
		if ( $word =~ /^[a-z']+$/ ) { # /[a-z][a-z]/ ) {
		    $unknown_roots{ $word } = 1;
		}
	    }
            $outline .= $word;
	} ## end for each word on nonblank line

	$outline = gzb_ascii_unicode_convert( $outline );
	$outline =~ s/__//g;
	
	$outline =~ s/^\t(.*)/$1 <br>/; # lines of poetry

        print GLOSSFRAME $outline . "\n";

    } # end if line is not blank
} # end for each line of input file

# write HTML footer to text frame

my $datestring = localtime();
print GLOSSFRAME<<FOOTER;
<HR />

<p>
<a href="gzb.htm" target="_top">Main gzb index</a>
<br />
Last updated on $datestring
</p>

</body>
</html>

FOOTER

# close files
close;

# write done message
print "done processing $infile\n";

my $reportfile = $infile;
$reportfile =~ s/.txt$/_unknown_words.rdb/;
if ( $reportfile !~ /rdb$/ ) {
    $reportfile .= "_unknown_words.rdb";
}


###TODO port improvements here back to htmlize-with-tables-and-interlinear-glosses.pl
# or better, put this stuff in a .pm module and call from each script
my $sec = 0;
my $min = 0;
my $hour = 0;
my $mday = 0;
my $month = 0;
my $year = 0;
my $wday = 0;
my $yday = 0;
my $isdist = 0;
($sec, $min, $hour, $mday, $month, $year, $wday, $yday, $isdist) = localtime(time);

$year += 1900;
$month += 1;

###TODO format month and mday with leading zeroes if one digit
my $datestr = "$year-$month-$mday";

my @most_productive_affixes = ( "la", "kxa", "hqa", "nxa", "ba",
				"zox", "van", "ca", "moxj" );

###TODO filter by stripping proper names
my @u = sort keys %unknown_roots;
if ( my $n = scalar @u ) {
    print STDERR "found $n root words in glosses that are not in lexicon, writing them to $reportfile\n";
    open REPORT, ">>$reportfile" or die "can't open $reportfile for writing";
    for ( my $i = 0; $i < $n;  $i++ ) {
	print REPORT $u[ $i ] . "\tr\t\t\?\t$datestr\n";
    }
    close REPORT;
}

###TODO filter variant verb forms of known stems
@u = sort keys %unknown_compounds;
if ( scalar @u ) {
    my @u = &filter_obvious_derivations( @u );
    if ( my $n = scalar @u ) {
	print STDERR "found $n compound words in glosses that are not in lexicon, writing them to $reportfile\n";
	open REPORT, ">>$reportfile" or die "can't open $reportfile for writing";
	for ( my $i = 0; $i < $n;  $i++ ) {
	    print REPORT $u[ $i ] . "\td\t\t\?\t$datestr\n";
	}
	close REPORT;
    }
}

# ====

my @most_productive_affixes = ( "la", "kxa", "hqa", "nxa", "ba",
				"zox", "van", "ca", "moxj" );
my @name_affixes = ( "ram", "sqam", "lam", "gam", "kxam" );

sub filter_obvious_derivations( @ ) {
    my @rc;
    my $already = 0;
    foreach ( @_ ) {
	my $word = $_;
	my $already = 0;
	foreach ( @most_productive_affixes ) {
	    my $t = $word;
	    $t =~ s/-$_(^[a-z])/$1/;
	    $t =~ s/-$_$//;
	    if ( grep { $t eq $_ } @lexicon_words ) {
		$already = 1;
		last;
	    }
	}
	foreach (  @name_affixes ) {
	    if ( $word =~ m/-$_$/ ) {
		$already = 1;
		last;
	    }
	}
	if ( not $already ) {
	    push @rc, $word;
	}
    }
    return @rc;
}


# ===

sub make_hyperlink( $ ) {
    $linkcount++;
    my $word = shift;
    my $anchor = $word;
    $anchor =~ s/'//g;
    $anchor =~ s/x/__x/g;
    $anchor =~ s/q/__q/g;
    my $link = qq(<a href="n__xcgt__x.htm#$anchor" target="lexicon">$word</a>);
    return $link;
}

#================

sub usage {
    print STDERR<<USAGE;
Usage: $0 <input text file> -o <output filename base> -t <document title>
-o	Base of output filenames; .html and _f.html will be appended to this.
	If not supplied, the input filename with .txt stripped off will be used.
-t	String to put in <TITLE> element of main document.
-l 	Lexicon input file.  Defaults to $lexicon
USAGE
}

#================

sub write_frameset( $$ ) {
    my $framesetfile = shift;
    my $topframefile = shift;

    open FRAMESET, "> $framesetfile"  or die "can't open $framesetfile for writing\n";

    print FRAMESET<<END;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN"
   "http://www.w3.org/TR/html4/frameset.dtd">
<HTML>
<HEAD>
<meta name="generator" content="gen-glossed-doc-frameset.pl">
<title>
$title
</title>
</HEAD>

<FRAMESET rows="80%, 20%">
      <FRAME src="$topframefile"    name="text">
      <FRAME src="$bottomframefile" name="lexicon">
</FRAMESET>
<NOFRAMES>
      <P>Since your browser doesn't support frames, click here:
      <br />
      <a href="$topframefile">$topframefile</a>
      </P>
</NOFRAMES>
</FRAMESET>
</HTML>

END

    close FRAMESET;
}
