# Convert input plain text to HTML.  Specialized for handling mixed
# normal text and interlinear glosses, which turn into borderless
# tables in the output.


# (c) 2004-2007 Jim Henry.  Creative Commons licensing


BEGIN {
  print "<html>";
  print "<head>";
  print "<title>"; # to be filled in manually by user
  print "glosses generated from " FILENAME;
  print "</title>";

  print "<meta name=\"generator\" content=\"htmlize-by-para-with-tables.awk\">";

  # this line is because of the Internet Explorer bug where some unicode chars
  # (e.g. IPA symbols) won't display unless the page stylesheet specifies a unicode font.
  print "<style><!--";
  print "BODY { font-family: \"Lucida Sans Unicode\", sans-serif }";
  print "--></style>"
  
  print "<body>";

  # pseudo-enum
  p = 1;
  pre = 2;
  table = 3;
  hr = 4;
  h1 = 10;
  h2 = 11;
  ul = 12;

  deflist = 1;
  interlinear = 2;
  first = 1;
  section = 0;

  srand();
  GID = int( rand() * 10000 ) ;

  tablestart = -99;
}

# on blank line
/^[\t ]*$/ {
  if ( in_para ) {
    if ( tag == p ) {
      print "</p>";
    } else if ( tag == table ) {
      print "</table>";
      if ( tabletype == interlinear ) {
	print "</blockquote>";
      }
    } else if ( tag == h1 ) {
      print "</h1>";
    } else if ( tag == h2 ) {
      print "</h2>";
    } else if ( tag == ul ) {
      print "</ul>";
    }
  }
  last_para_type = tag;
  in_para = 0;
  tableline = -1;
}

# on nonblank line
! /^ *$/ {
  if ( 0 == in_para ) {
    in_para = 1;

    table_anchor = "t" (++para_count) "_" GID;

    # a dashes line becomes a horizontal rule element
    if ( $0 ~ /^----+$/ ) { 
      print "<hr>";
      tag = hr;
    } else if ( $0 ~ /^\#/ ) {
      print "<ul>";
      tag = ul;
    } else {
      if ( first ) {
	first = 0;
	print "<H1>";
	tag = h1;
      } else {
	print "<p>";
	tag = p;
      }
    }
  }
  gsub ( "&", "\\&amp;" );
  gsub ( "\"", "\\&quot;" );
  gsub ( "<", "\\&lt;" );
  gsub ( ">", "\\&gt;" );
  gsub ( " -- ", " \\&mdash; " );
  gsub ( "-- ", "\\&mdash; " );
  gsub ( " --", " \\&mdash;" );

  # intended for working on one or more emphasized words in interlinear gloss.
  # needs more  sophistication to work across multiple lines
  while ( sub( /\%\% */, "<strong>" ) ) {
    sub( / *\%\%/, "</strong>" );
  }

  # this wants more sophistication to work across multiple lines
  if ( sub ( "_", "<cite>" ) ) {
     sub ( "_", "</cite>" )
  }
}

/^link:/ {
  url = $2;
  if ( $3 != "" ) {
    # kludgey, limited, need to find better way
    # would be trivial with join if convert this whole script to perl
    linkword = $3 " " $4 " " $5  " " $6  " " $7 " " $8 " " $9;
    sub( / +$/, "", linkword );
  } else {
    linkword = url;
  }
  sub( /^link:.*/, "<a href=\"" url "\">" linkword "</a>" );
}

{
  if ( in_para && tag == table ) {
    printf "<tr>";
    if ( tabletype == interlinear ) {
      # glossed language text.  Link each morpheme to dictionary.
      if ( NR == tablestart ) {
	for ( i = 1; i <= NF; i++ ) {

	  ##### If you're modifying this for your own conlang
	  ##### you probably want to uncomment  the next line
	  ##### and comment out the remainder of this for loop.
	  ##### Or you could modify the other code as necessary
	  ##### to link to your own dictionary file.

          #printf "<td> " $i " </td> ";

	  printf "<td> ";
	  morphcount = split( $i, morphemes, "-" );

	  this_word = $i;
	  gsub ( /[\]\[\.\?\:\!\*}{]/, "", this_word );
	  print this_word "\t" table_anchor > "/dev/stderr";

	  # split() produces 1-based array, not 0-based
	  for ( j = 1; j <= morphcount; j++ ) {
	    this_morpheme = morphemes[j];
	    gsub( /<\/*strong>/, "", this_morpheme );
	    gsub ( /[\]\[\.\?\:\!\*}{]/, "", this_morpheme );
	    print this_morpheme  "\t"  table_anchor   > "/dev/stderr";

	    if ( j >= 2 ) {
	      printf "-";
	    }
	    anchor = this_morpheme;
	    gsub( /x/, "__x", anchor );
	    gsub( /q/, "__q", anchor );
	    gsub ( /[\'\]\[\.\?\:\!\*}{]/, "", anchor );

	    printf "<a href=\"n__xcgt__x.htm#" anchor "\">"  morphemes[j] "</a>";
	  }
	  printf " </td> ";
	}
	fields_last_line = NF;
      } else if ( NR == tablestart + 1 ) {
	for ( i = 1; i <= NF; i++ ) {
	  printf "<td> " $i " </td> ";
	}
	fields_last_line = NF;
      } else if ( NR == tablestart + 2 ) {
	print "<td colspan=\"" fields_last_line "\">" $0 "</td>";
      } else {
	print "line " NR ": warning: thought this para was interlinear gloss due to spacing," > "/dev/stderr";
	print "but it goes on >3 lines" > "/dev/stderr";
	print "<td colspan=\"" fields_last_line "\">" $0 "</td>";
      }
    } else {
      sub( /[\t ]+$/, "" );
      gsub( /\t+ */, " \t<td> " );
      line = $0;
      if ( $0 !~ /^\<td/ ) {
	line = "<td> " $0;
      }
      printf line;
    } 
      
    printf " </tr>\n";
  } else if ( tag == hr ) {
    gsub( /----+/, "" );
    print;
  } else if ( tag == ul ) {
    sub( /\#/, "<li>" );
    print;
  } else {
    print;
  }
}

END {

  if ( in_para ) {
    if ( tag == p ) {
      print "</p>";
    } else if ( tag == table ) {
      print "</table>";
    } else if ( tag == h1 ) {
      print "</h1>";
    } else if ( tag == h2 ) {
      print "</h2>";
    }
  }
  print "\n</body></html>"
}

