#! /usr/bin/perl -w

# Calculate the index of synthesis for gjax-zym-byn
# Count the words in the corpus and the total number of 
# morphemes, and divide the latter by the former

use strict;
use warnings;

my $debug = 0;

sub morpheme_count {
    my $word = shift;
    my $morphs = 0;
    if ( $word =~ m/-/ ) {
	my @emes = split /-/, $word;
	foreach ( @emes ) {
	     $morphs += morpheme_count( $_ );
	}
	return $morphs;
    } else {
	if ( $word =~ m/^[a-z][xq]?(o|i|rq)$/ ) {
	    print "'si' type postp: $word\n"		if $debug;
	    return 2;
	} elsif ( $word =~ m/^[a-z][xq]?(o|i|rq)[jlrmn]x?$/ ) {
	    print "'sin' type postp: $word\n"		if $debug;
	    return 3;
	} elsif ( $word =~ m/^(o|i|rq)[jlrmn]x?$/ ) {
	    print "'im' type postp: $word\n"		if $debug;
	    return 2;
	# this condition has a lot of false positives e.g. {geq'diqm}
#	} elsif ( $word =~ m/eq'[a-z][xq]?(o|i|rq)/ ) {
	# this fixes that:
	} elsif ( $word =~ m/eq'[a-z][xq]?(o|i|rq)$/
	       or $word =~ m/eq'[a-z][xq]?(o|i|rq)[jlrmn]x?$/) {
	    print "'beq'gij' type postp: $word\n"	if $debug;
	    my @emes = split /eq\'/, $word;
	    foreach ( @emes ) {
		$morphs += morpheme_count( $_ );
	    }
	    return $morphs;
	} else {
	    print "no special cases found for $word\n"	if $debug;
	    return 1;
	}
    }
}

my $words = 0;
my $morphemes = 0;

# slurp one or more text files in gzb-ASCII
while ( <> ) {
    s/http[^\s]+/ /;
    my @words_this_line = split /[^a-zA-Z\'-]+/;
    foreach ( @words_this_line ) {
	if ( $_ ne "" ) {
	    $words++;
	    $morphemes += morpheme_count( $_ );
	}
    }
}

print "$words words containing $morphemes morphemes, index of synthesis = " 
    . ( $morphemes / $words ) . "\n";

    

