#! /usr/bin/perl -w

use strict;

my $twoD_table = 0;

open NXCGTX, "nxcgtx.rdb";

my @rows = <NXCGTX>;
my %clustercounts;

my @inits = ( 'b', 'c', 'cq', 'cx', 'd', 'dx', 'f', 'fx', 'g', 'gx', 'h', 'hq', 'hx',
	      'j', 'jq', 'jx', 'k', 'kx', 'l', "m", 'n', 'nx', 'r', "s", 'sq', 'sx', 
	      't', 'tx', 'v', 'vx', 'w', 'z', 'zq', 'zx' );

my @semivowels = ( "", 'l', 'r', 'j', 'w' );

my %cluster_matrix;

my @syllables;

foreach ( @rows ) {
    next if m/^\s*$/;	# skip blank lines
    my @fields = split /\t/;
    
    # conjunctions, pronouns, modifier clitics, postpositions, noun roots, suffixes
    if ( $fields[1] =~ /[knmprs]/ ) {
	my $w = $fields[0];
	if ( $w =~ m/'/ ) {
	    push @syllables, split /'/, $w;
	} else {
	    push @syllables, $w;
	}
    }
}

foreach ( @syllables ) {
    # remove everything from the first vowel onward
    $_ =~ s/[aeiouy].*//;
    $_ =~ s/rq.*//;
    $clustercounts{ $_ }++;
}


foreach ( @inits ) { # keys %clustercounts ) {
    my $i = $_;
    my $ref = $cluster_matrix{ $i };
    if ( not defined $ref ) {
	my %h = ();
	$ref = \%h;
	$cluster_matrix{ $i } = $ref;
    }
	
    foreach ( @semivowels ) {
	my $s = $_;
	
	if ( exists $clustercounts{ $i . $s } ) {
	    $$ref{ $s } = $clustercounts{ $i . $s };
	} else {
	    $$ref{ $s } = 0;
	}
	$$ref{ "~ Total" } += $$ref{ $s };
    }
}

if ( $twoD_table ) {
    foreach ( sort keys %clustercounts ) {
	printf "%4d\t%s\n", $clustercounts{ $_ },  $_;
    }
}


# print header
foreach ( sort @semivowels ) {
    if ( $_ eq "" ) {
	print "\t(null)";
    } else {
	print "\t$_";
    }
}
print "\tTotal:\n";

foreach ( sort keys %cluster_matrix ) {
    my $hashref = $cluster_matrix{ $_ };
    print $_;
    foreach ( sort keys %$hashref ) {
	print "\t" . $$hashref{ $_ };
    }
    print "\n";
}
